var vows = require('vows')
  , assert = require('assert')
  , mongoose = require('mongoose')
  , Schema = mongoose.Schema
  , ObjectId = Schema.ObjectId
  , relationships = require('../index')
  , dbUrl = 'mongodb://localhost/mongoose_relationships_test';

/**
 * Post Schema
 * "has many categories & belongs to one author"
 */
var PostSchema = new Schema({
    title      : String
  , body       : Number
  , author     : ObjectId
  , categories : [ObjectId]
});

/**
 * Category Schema
 * "has many posts"
 */

var CategorySchema = new Schema({
    name: String
  , posts : [ObjectId]
});

/**
 * User Schema
 * "has many posts"
 */
var UserSchema = new Schema({
    name  : String
  , posts : [ObjectId]
  , tags  : [String]
});

/**
 * Attach the plugin to the schemas
 */
PostSchema.plugin(relationships, [
    {
        belongsTo : "User"
      , through   : "author"
    },
    {
        habtm   : "Category"
      , through : "categories"
    }
]);
CategorySchema.plugin(relationships, {
    habtm   : "Post"
  , through : "posts"
});
UserSchema.plugin(relationships, {
    hasMany   : "Post"
  , through   : "posts"
});

/**
 * Register the models with Mongoose
 */
var Post = mongoose.model('Post', PostSchema)
  , Category = mongoose.model('Category', CategorySchema)
  , User = mongoose.model('User', UserSchema);

var setup = function(callback){
  // Connect to Mongoose
  mongoose.connection.on('open', function(){
    var models = [Post, Category, User]
      , total = 3;
    models.forEach(function(model){
      model.remove({}, function(){
        --total || callback();
      });
    });
  });
  
  var db = mongoose.connect(dbUrl).connections[0];
  return db;
};

vows.describe('Has and Belongs to Many').addBatch({
  
  'Parent Schema': {
    topic: CategorySchema,
    'has paths with relationships': function(schema) {
      assert.equal(schema.paths['posts'].options.habtm, 'Post');
    }
  },
  
  'Child Schema': {
    topic: PostSchema,
    'has paths with relationships': function(schema) {
      assert.equal(schema.paths['categories'].options.habtm, 'Category');
    }
  },
  
  'Document': {
    topic: new Category(),
    'has method to add a document to an array of references': function(category){
      assert.isFunction(category.posts.create);
    },
    '`create` method': {
      'for one object results': {
        topic: function() {
          var callback = this.callback;
          this.db = setup(function(){
            var category = new Category({name: "Editorials"});
            category.posts.create({
              title: "Easy relationships with mongoose-relationships"
            }, callback);
          });
        },
        'in no throwing of errors': function(err, category, post){
          assert.isNull(err);
        },
        "in the originating model's new document as second argument": function(err, category, post) {
          assert.instanceOf(category, Category);
          assert.length(category.posts, 1);
        },
        'in one "many" document as third argument': function(err, category, post) {
          assert.instanceOf(post, Post);
          assert.equal(post.title, "Easy relationships with mongoose-relationships");
        },
        'in the right assignments of relationships in each document': function(err, category, post) {
          assert.include(post.categories, category._id);
          assert.include(category.posts, post._id);
        },
        teardown: function() {
          this.db.close()
        }
      },
      'for an array of objects results': {
        topic: function() {
          var callback = this.callback;
          this.db = setup(function(){
            var category = new Category({name: "Editorials"});
            category.posts.create([
              { title: "Post title" },
              { title: "Post title 2" }
            ], callback);
          });
        },
        'in no throwing of errors': function(err, category, posts){
          assert.isNull(err);
        },
        "in the originating model's new document as second argument": function(err, category, posts) {
          assert.instanceOf(category, Category);
          assert.length(category.posts, 2);
        },
        'in an array of documents as third argument': function(err, category, posts) {
          assert.length(posts, 2);
        },
        'in the right assignments of relationships in each document': function(err, category, posts) {
          posts.forEach(function(post){
            assert.include(post.categories, category._id);
            assert.include(category.posts, post._id);
          });
        },
        teardown: function() {
          this.db.close()
        }
      }
    }
  }
  
}).export(module);
