var vows = require('vows')
  , assert = require('assert')
  , mongoose = require('mongoose')
  , Schema = mongoose.Schema
  , ObjectId = Schema.ObjectId
  , relationships = require('../index')
  , dbUrl = 'mongodb://localhost/mongoose_relationships_test';

/**
 * Blog Post Schema
 * "has many categories & belongs to one author"
 */
var PostSchema = new Schema({
    title      : String
  , body       : Number
  , author     : ObjectId
});

/**
 * User Schema
 * "has many posts"
 */
var UserSchema = new Schema({
    name  : String
  , posts : [ObjectId]
  , tags  : [String]
});

/**
 * Attach the plugin to the schemas
 */
PostSchema.plugin(relationships, {
    belongsTo : "User"
  , through   : "author"
});
UserSchema.plugin(relationships, {
    hasMany   : "Post"
  , through   : "posts"
});

/**
 * Register the models with Mongoose
 */
var Post = mongoose.model('Post', PostSchema)
  , User = mongoose.model('User', UserSchema);

var setup = function(callback){
  // Connect to Mongoose
  mongoose.connection.on('open', function(){
    var models = [Post, User]
      , total = 2;
    models.forEach(function(model){
      model.remove({}, function(){
        --total || callback();
      });
    });
  });
  
  var db = mongoose.connect(dbUrl).connections[0];
  return db;
};

vows.describe('One to Many').addBatch({
  
  'Parent Schema': {
    topic: UserSchema,
    'has paths with relationships': function(schema) {
      assert.equal(schema.paths['posts'].options.hasMany, 'Post');
    }
  },
  
  'Child Schema': {
    topic: PostSchema,
    'has paths with relationships': function(schema) {
      assert.equal(schema.paths['author'].options.belongsTo, 'User');
    }
  },
  
  'Child Document': {
    topic: new User(),
    'has method to add a document to an array of references': function(user){
      assert.isFunction(user.posts.create);
    },
    '`create` method': {
      'for one object results': {
        topic: function() {
          var callback = this.callback;
          this.db = setup(function(){
            var user = new User({name: "John Smith"});
            user.posts.create({
              title: "Easy relationships with mongoose-relationships"
            }, callback);
          });
        },
        'in no throwing of errors': function(err, user, post){
          assert.isNull(err);
        },
        "in the originating model's new document as second argument": function(err, user, post) {
          assert.instanceOf(user, User);
          assert.length(user.posts, 1);
        },
        'in one "many" document as third argument': function(err, user, post) {
          assert.instanceOf(post, Post);
          assert.equal(post.title, "Easy relationships with mongoose-relationships");
        },
        'in the right assignments of relationships in each document': function(err, user, post) {
          assert.equal(post.author.toString(), user._id.toString());
          assert.equal(user.posts[0].toString(), post._id.toString());
        },
        teardown: function() {
          this.db.close()
        }
      },
      'for an array of objects results': {
        topic: function() {
          var callback = this.callback;
          this.db = setup(function(){
            var user = new User({name: "John Smith"});
            user.posts.create([
              { title: "Post title" },
              { title: "Post title 2" }
            ], callback);
          });
        },
        'in no throwing of errors': function(err, user, posts){
          assert.isNull(err);
        },
        "in the originating model's new document as second argument": function(err, user, posts) {
          assert.instanceOf(user, User);
          assert.length(user.posts, 2);
        },
        'in an array of documents as third argument': function(err, user, posts) {
          assert.length(posts, 2);
        },
        'in the right assignments of relationships in each document': function(err, user, posts) {
          posts.forEach(function(post, i){
            assert.equal(post.author.toString(), user._id.toString());
            assert.equal(user.posts[i].toString(), post._id.toString());
          });
        },
        teardown: function() {
          this.db.close()
        }
      }
    }
  }
  
}).export(module);
