var vows = require('vows')
  , assert = require('assert')
  , mongoose = require('mongoose')
  , Schema = mongoose.Schema
  , ObjectId = Schema.ObjectId
  , relationships = require('../index')
  , dbUrl = 'mongodb://localhost/mongoose_relationships_test';

/**
 * Blog Schema
 * "belongs to one editor"
 */
var BlogSchema = new Schema({
    title      : String
  , editor     : ObjectId
});

/**
 * User Schema
 * "has one blog"
 */
var UserSchema = new Schema({
    name : String
  , blog : ObjectId
});

/**
 * Attach the plugin to the schemas
 */
BlogSchema.plugin(relationships, {
    belongsTo : "User"
  , through   : "editor"
});
UserSchema.plugin(relationships, {
    hasOne    : "Blog"
  , through   : "blog"
});

/**
 * Register the models with Mongoose
 */
var Blog = mongoose.model('Blog', BlogSchema)
  , User = mongoose.model('User', UserSchema);

var setup = function(callback){
  // Connect to Mongoose
  mongoose.connection.on('open', function(){
    var models = [Blog, User]
      , total = 2;
    models.forEach(function(model){
      model.remove({}, function(){
        --total || callback();
      });
    });
  });
  
  var db = mongoose.connect(dbUrl).connections[0];
  return db;
};

vows.describe('One to One').addBatch({
  
  'Parent Schema': {
    topic: UserSchema,
    'has paths with relationships': function(schema) {
      assert.equal(schema.paths['blog'].options.hasOne, 'Blog');
    }
  },
  
  'Child Schema': {
    topic: BlogSchema,
    'has paths with relationships': function(schema) {
      assert.equal(schema.paths['editor'].options.belongsTo, 'User');
    }
  },
  
  'Child Document': {
    topic: new User(),
    'has method to add a document to a reference field': function(user){
      assert.isFunction(user.blog.create);
    },
    '`create` method': {
      'for one object results': {
        topic: function() {
          var callback = this.callback;
          this.db = setup(function(){
            var user = new User({name: "John Smith"});
            user.blog.create({
              title: "Blog about mongoose-relationships"
            }, callback);
          });
        },
        'in no throwing of errors': function(err, user, blog){
          assert.isNull(err);
        },
        "in the originating model's new document as second argument": function(err, user, blog) {
          assert.instanceOf(user, User);
          assert.isNotNull(user.blog, 1);
        },
        'in one child document as third argument': function(err, user, blog) {
          assert.instanceOf(blog, Blog);
          assert.equal(blog.title, "Blog about mongoose-relationships");
        },
        'in the right assignments of relationships in each document': function(err, user, blog) {
          assert.equal(blog.editor.toString(), user._id.toString());
          assert.equal(user.blog.toString(), blog._id.toString());
        },
        teardown: function() {
          this.db.close()
        }
      },
      'for an array of objects results': {
        topic: function() {
          var callback = this.callback;
          this.db = setup(function(){
            var user = new User({name: "John Smith"});
            user.blogs.create([
              { title: "Blog title" },
              { title: "Blog title 2" }
            ], callback);
          });
        },
        'in the throwing of errors': function(err, user, blogs){
          assert.isNotNull(err);
        },
        teardown: function() {
          this.db.close()
        }
      }
    }
  }
  
}).export(module);
